@@ -25,7 +25,7 @@ pub enum LintOrTidyCommandType {
 }
 
 impl LintOrTidyCommandType {
-    fn what(&self) -> &'static str {
+    fn what(self) -> &'static str {
         match self {
             LintOrTidyCommandType::Lint => "linter",
             LintOrTidyCommandType::Tidy => "tidier",
@@ -48,20 +48,46 @@ impl fmt::Display for LintOrTidyCommandType {
 pub enum Invoke {
     #[serde(rename = "per-file")]
     PerFile,
+    #[serde(rename = "per-file-or-dir")]
+    PerFileOrDir(usize),
+    #[serde(rename = "per-file-or-once")]
+    PerFileOrOnce(usize),
     #[serde(rename = "per-dir")]
     PerDir,
+    #[serde(rename = "per-dir-or-once")]
+    PerDirOrOnce(usize),
     #[serde(rename = "once")]
     Once,
 }
 
 impl fmt::Display for Invoke {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        // Is using serde to do this incredibly gross?
-        f.write_str(match self {
-            Invoke::PerFile => r#""per-file""#,
-            Invoke::PerDir => r#""per-dir""#,
-            Invoke::Once => r#""once""#,
-        })
+        match self {
+            Invoke::PerFile => write!(f, r#"invoke = "per-file""#),
+            Invoke::PerFileOrDir(n) => write!(f, "invoke.per-file-or-dir = {n}"),
+            Invoke::PerFileOrOnce(n) => write!(f, "invoke.per-file-or-once = {n}"),
+            Invoke::PerDir => write!(f, r#"invoke = "per-dir""#),
+            Invoke::PerDirOrOnce(n) => write!(f, "invoke.per-dir-or-once = {n}"),
+            Invoke::Once => write!(f, r#"invoke = "once""#),
+        }
+    }
+}
+
+#[derive(Clone, Copy, Debug, Deserialize, Eq, PartialEq, Serialize)]
+pub enum ActualInvoke {
+    PerFile,
+    PerDir,
+    Once,
+}
+
+impl ActualInvoke {
+    #[cfg(test)]
+    fn as_invoke(&self) -> Invoke {
+        match *self {
+            ActualInvoke::PerFile => Invoke::PerFile,
+            ActualInvoke::PerDir => Invoke::PerDir,
+            ActualInvoke::Once => Invoke::Once,
+        }
     }
 }
 
@@ -78,7 +104,7 @@ impl fmt::Display for WorkingDir {
             WorkingDir::Root => f.write_str(r#""root""#),
             WorkingDir::Dir => f.write_str(r#""dir""#),
             WorkingDir::ChdirTo(cd) => {
-                f.write_str(r#"chdir_to = ""#)?;
+                f.write_str(r#"chdir-to = ""#)?;
                 f.write_str(&format!("{}", cd.display()))?;
                 f.write_str(r#"""#)
             }
@@ -118,7 +144,7 @@ impl fmt::Display for PathArgs {
 #[derive(Debug, Error, PartialEq, Eq)]
 enum CommandError {
     #[error(
-        "You cannot create a Command which lints and tidies without lint_flags and/or tidy_flags"
+        "You cannot define a command which lints and tidies without lint-flags and/or tidy-flags"
     )]
     CommandWhichIsBothRequiresLintOrTidyFlags,
 
@@ -137,11 +163,13 @@ enum CommandError {
 }
 
 #[derive(Debug)]
+#[allow(clippy::module_name_repetitions)]
 pub struct LintOrTidyCommand {
     project_root: PathBuf,
     pub name: String,
     typ: LintOrTidyCommandType,
     includer: Matcher,
+    include: Vec<String>,
     excluder: Matcher,
     invoke: Invoke,
     working_dir: WorkingDir,
@@ -228,18 +256,19 @@ impl LintOrTidyCommand {
                 params
                     .ignore_stderr
                     .into_iter()
-                    .map(|i| Regex::new(&i).map_err(|e| e.into()))
+                    .map(|i| Regex::new(&i).map_err(Into::into))
                     .collect::<Result<Vec<_>>>()?,
             )
         };
 
-        let cmd = replace_root(params.cmd, &params.project_root);
+        let cmd = replace_root(&params.cmd, &params.project_root);
         let root = params.project_root.clone();
         Ok(LintOrTidyCommand {
             project_root: params.project_root,
             name: params.name,
             typ: params.typ,
             includer: MatcherBuilder::new(&root).with(&params.include)?.build()?,
+            include: params.include,
             excluder: MatcherBuilder::new(&root).with(&params.exclude)?.build()?,
             invoke: params.invoke,
             working_dir: params.working_dir,
@@ -286,27 +315,96 @@ impl LintOrTidyCommand {
     // This returns a vec of vecs where each of the sub-vecs contains 1+
     // files. Each of those sub-vecs represents one invocation of the
     // program. The exact paths that are passed to that invocation are later
-    // determined based on the command's `path_args` field.
-    pub fn files_to_args_sets<'a>(&self, files: &'a [PathBuf]) -> Result<Vec<Vec<&'a Path>>> {
+    // determined based on the command's `path-args` field.
+    pub fn files_to_args_sets<'a>(
+        &self,
+        files: &'a [PathBuf],
+    ) -> Result<(Vec<Vec<&'a Path>>, ActualInvoke)> {
         let files = files.iter().filter(|f| self.file_matches_rules(f));
         Ok(match self.invoke {
             // Every file becomes its own one one-element Vec.
-            Invoke::PerFile => files.sorted().map(|f| vec![f.as_path()]).collect(),
+            Invoke::PerFile => (
+                files.sorted().map(|f| vec![f.as_path()]).collect(),
+                ActualInvoke::PerFile,
+            ),
+            Invoke::PerFileOrDir(n) => {
+                let count = files.clone().count();
+                if count < n {
+                    debug!(
+                        "Invoking {} once per file for {count} files, which is less than {n}.",
+                        self.name,
+                    );
+                    (
+                        files.sorted().map(|f| vec![f.as_path()]).collect(),
+                        ActualInvoke::PerFile,
+                    )
+                } else {
+                    debug!(
+                        "Invoking {} once per directory for {count} files, which is at least {n}.",
+                        self.name,
+                    );
+                    (Self::files_to_dirs(files)?, ActualInvoke::PerDir)
+                }
+            }
+            Invoke::PerFileOrOnce(n) => {
+                let count = files.clone().count();
+                if count < n {
+                    debug!(
+                        "Invoking {} once per file for {count} files, which is less than {n}.",
+                        self.name,
+                    );
+                    (
+                        files.sorted().map(|f| vec![f.as_path()]).collect(),
+                        ActualInvoke::PerFile,
+                    )
+                } else {
+                    debug!(
+                        "Invoking {} once for all {count} files, which is at least {n}.",
+                        self.name,
+                    );
+                    (
+                        vec![files.sorted().map(PathBuf::as_path).collect()],
+                        ActualInvoke::Once,
+                    )
+                }
+            }
             // Every directory becomes a Vec of its files.
-            Invoke::PerDir => {
-                let files = files.map(|p| p.as_ref()).collect::<Vec<_>>();
-                let by_dir = Self::files_by_dir(&files)?;
-                by_dir
-                    .into_iter()
-                    .sorted_by_key(|(k, _)| *k)
-                    .map(|(_, v)| v.into_iter().sorted().collect())
-                    .collect()
+            Invoke::PerDir => (Self::files_to_dirs(files)?, ActualInvoke::PerDir),
+            Invoke::PerDirOrOnce(n) => {
+                let dirs = Self::files_to_dirs(files.clone())?;
+                let count = dirs.len();
+                if count < n {
+                    debug!("Invoking {} once per directory because there are fewer than {n} directories.", self.name);
+                    (dirs, ActualInvoke::PerDir)
+                } else {
+                    debug!(
+                        "Invoking {} once for all {count} directories, which is at least {n}.",
+                        self.name,
+                    );
+                    (
+                        vec![files.sorted().map(PathBuf::as_path).collect()],
+                        ActualInvoke::Once,
+                    )
+                }
             }
             // All the files in one Vec.
-            Invoke::Once => vec![files.sorted().map(PathBuf::as_path).collect()],
+            Invoke::Once => (
+                vec![files.sorted().map(PathBuf::as_path).collect()],
+                ActualInvoke::Once,
+            ),
         })
     }
 
+    fn files_to_dirs<'a>(files: impl Iterator<Item = &'a PathBuf>) -> Result<Vec<Vec<&'a Path>>> {
+        let files = files.map(AsRef::as_ref).collect::<Vec<_>>();
+        let by_dir = Self::files_by_dir(&files)?;
+        Ok(by_dir
+            .into_iter()
+            .sorted_by_key(|(k, _)| *k)
+            .map(|(_, v)| v.into_iter().sorted().collect())
+            .collect())
+    }
+
     fn files_by_dir<'a>(files: &[&'a Path]) -> Result<HashMap<&'a Path, Vec<&'a Path>>> {
         let mut by_dir: HashMap<&Path, Vec<&Path>> = HashMap::new();
         for f in files {
@@ -318,18 +416,22 @@ impl LintOrTidyCommand {
         Ok(by_dir)
     }
 
-    pub fn tidy(&self, files: &[&Path]) -> Result<Option<TidyOutcome>> {
+    pub fn tidy(
+        &self,
+        actual_invoke: ActualInvoke,
+        files: &[&Path],
+    ) -> Result<Option<TidyOutcome>> {
         self.require_is_not_command_type("tidy", LintOrTidyCommandType::Lint)?;
 
-        if !self.should_act_on_files(files)? {
+        if !self.should_act_on_files(actual_invoke, files)? {
             return Ok(None);
         }
 
-        let path_metadata = self.maybe_path_metadata_for(files)?;
+        let path_metadata = self.maybe_path_metadata_for(actual_invoke, files)?;
 
         let in_dir = self.in_dir(files[0])?;
         let operating_on = self.operating_on(files, &in_dir)?;
-        let mut cmd = self.command_for_paths(&self.tidy_flags, &operating_on)?;
+        let mut cmd = self.command_for_paths(&self.tidy_flags, &operating_on);
 
         info!(
             "Tidying [{}] with {} in [{}] using command [{}]",
@@ -342,7 +444,7 @@ impl LintOrTidyCommand {
         let bin = cmd.remove(0);
         exec::run(
             &bin,
-            &cmd.iter().map(|c| c.as_str()).collect::<Vec<_>>(),
+            &cmd.iter().map(String::as_str).collect::<Vec<_>>(),
             &self.env,
             &self.ok_exit_codes,
             self.ignore_stderr.as_deref(),
@@ -358,16 +460,20 @@ impl LintOrTidyCommand {
         Ok(Some(TidyOutcome::Unknown))
     }
 
-    pub fn lint(&self, files: &[&Path]) -> Result<Option<LintOutcome>> {
+    pub fn lint(
+        &self,
+        actual_invoke: ActualInvoke,
+        files: &[&Path],
+    ) -> Result<Option<LintOutcome>> {
         self.require_is_not_command_type("lint", LintOrTidyCommandType::Tidy)?;
 
-        if !self.should_act_on_files(files)? {
+        if !self.should_act_on_files(actual_invoke, files)? {
             return Ok(None);
         }
 
         let in_dir = self.in_dir(files[0])?;
         let operating_on = self.operating_on(files, &in_dir)?;
-        let mut cmd = self.command_for_paths(&self.lint_flags, &operating_on)?;
+        let mut cmd = self.command_for_paths(&self.lint_flags, &operating_on);
 
         info!(
             "Linting [{}] with {} in [{}] using command [{}]",
@@ -380,7 +486,7 @@ impl LintOrTidyCommand {
         let bin = cmd.remove(0);
         let result = exec::run(
             &bin,
-            &cmd.iter().map(|c| c.as_str()).collect::<Vec<_>>(),
+            &cmd.iter().map(String::as_str).collect::<Vec<_>>(),
             &self.env,
             &self.ok_exit_codes,
             self.ignore_stderr.as_deref(),
@@ -410,11 +516,11 @@ impl LintOrTidyCommand {
         Ok(())
     }
 
-    fn should_act_on_files(&self, files: &[&Path]) -> Result<bool> {
-        match self.invoke {
-            Invoke::PerFile => {
+    fn should_act_on_files(&self, actual_invoke: ActualInvoke, files: &[&Path]) -> Result<bool> {
+        match actual_invoke {
+            ActualInvoke::PerFile => {
                 let f = &files[0];
-                // This check isn't stricly necessary since we default to not
+                // This check isn't strictly necessary since we default to not
                 // matching, but the debug output is helpful.
                 if self.excluder.path_matches(f, false) {
                     debug!(
@@ -433,7 +539,7 @@ impl LintOrTidyCommand {
                     return Ok(true);
                 }
             }
-            Invoke::PerDir => {
+            ActualInvoke::PerDir => {
                 let dir = files[0]
                     .parent()
                     .ok_or_else(|| CommandError::PathHasNoParent {
@@ -464,7 +570,7 @@ impl LintOrTidyCommand {
                     self.name
                 );
             }
-            Invoke::Once => {
+            ActualInvoke::Once => {
                 for f in files {
                     if self.excluder.path_matches(f, false) {
                         debug!(
@@ -508,12 +614,12 @@ impl LintOrTidyCommand {
                 .iter()
                 .sorted()
                 .map(|r| self.path_relative_to(r, in_dir))
-                .collect::<Result<Vec<_>>>()?),
-            PathArgs::Dir => Self::files_by_dir(files)?
+                .collect::<Vec<_>>()),
+            PathArgs::Dir => Ok(Self::files_by_dir(files)?
                 .into_keys()
                 .sorted()
                 .map(|r| self.path_relative_to(r, in_dir))
-                .collect::<Result<Vec<_>>>(),
+                .collect::<Vec<_>>()),
             PathArgs::None => Ok(vec![]),
             PathArgs::Dot => Ok(vec![PathBuf::from(".")]),
             PathArgs::AbsoluteFile => Ok(files
@@ -539,7 +645,7 @@ impl LintOrTidyCommand {
         }
     }
 
-    fn path_relative_to(&self, path: &Path, in_dir: &Path) -> Result<PathBuf> {
+    fn path_relative_to(&self, path: &Path, in_dir: &Path) -> PathBuf {
         let mut abs = self.project_root.clone();
         abs.push(path);
 
@@ -547,10 +653,10 @@ impl LintOrTidyCommand {
             if diff == Path::new("") {
                 diff = PathBuf::from(".");
             }
-            return Ok(diff);
+            return diff;
         }
 
-        Ok(path.to_path_buf())
+        path.to_path_buf()
     }
 
     // This takes the list of files relevant to the command. That list comes
@@ -559,14 +665,18 @@ impl LintOrTidyCommand {
     // determines what paths it should collect metadata for (which may be
     // none). This metadata is collected for tidy commands so we can determine
     // whether the command changed anything.
-    fn maybe_path_metadata_for(&self, files: &[&Path]) -> Result<Option<PathMetadata>> {
-        match self.invoke {
+    fn maybe_path_metadata_for(
+        &self,
+        actual_invoke: ActualInvoke,
+        files: &[&Path],
+    ) -> Result<Option<PathMetadata>> {
+        match actual_invoke {
             // If it's invoked per file we know that we only have one file in
             // `files`.
-            Invoke::PerFile => Ok(Some(self.path_metadata_for(files[0])?)),
+            ActualInvoke::PerFile => Ok(Some(self.path_metadata_for(files[0])?)),
             // If it's invoked per dir we can look at the first file's
             // parent. All the files should have the same dir.
-            Invoke::PerDir => {
+            ActualInvoke::PerDir => {
                 let dir = files[0]
                     .parent()
                     .ok_or_else(|| CommandError::PathHasNoParent {
@@ -577,7 +687,7 @@ impl LintOrTidyCommand {
             // If it's invoked once we would have to look at the entire
             // tree. That might be too expensive so we won't report a tidy
             // outcome in this case.
-            Invoke::Once => Ok(None),
+            ActualInvoke::Once => Ok(None),
         }
     }
 
@@ -643,11 +753,7 @@ impl LintOrTidyCommand {
         })
     }
 
-    fn command_for_paths(
-        &self,
-        flags: &Option<Vec<String>>,
-        paths: &[PathBuf],
-    ) -> Result<Vec<String>> {
+    fn command_for_paths(&self, flags: &Option<Vec<String>>, paths: &[PathBuf]) -> Vec<String> {
         let mut cmd = self.cmd.clone();
         if let Some(flags) = flags {
             for f in flags {
@@ -662,7 +768,36 @@ impl LintOrTidyCommand {
             cmd.push(p.to_string_lossy().to_string());
         }
 
-        Ok(cmd)
+        cmd
+    }
+
+    pub(crate) fn paths_summary(&self, actual_invoke: ActualInvoke, paths: &[&Path]) -> String {
+        let all = paths
+            .iter()
+            .sorted()
+            .map(|p| p.to_string_lossy().to_string())
+            .join(" ");
+        if paths.len() <= 3 {
+            return all;
+        }
+
+        match actual_invoke {
+            ActualInvoke::Once | ActualInvoke::PerDir => {
+                let initial = paths
+                    .iter()
+                    .sorted()
+                    .take(2)
+                    .map(|p| p.to_string_lossy())
+                    .join(" ");
+                format!(
+                    "{} files matching {}, starting with {}",
+                    paths.len(),
+                    self.include.join(" "),
+                    initial
+                )
+            }
+            ActualInvoke::PerFile => format!("{} files: {}", paths.len(), all),
+        }
     }
 
     fn paths_were_changed(&self, prev: PathMetadata) -> Result<bool> {
@@ -748,13 +883,13 @@ impl LintOrTidyCommand {
 
     pub fn config_debug(&self) -> String {
         format!(
-            "invoke = {} | working_dir = {} | path_args = {}",
+            "{} | working-dir = {} | path-args = {}",
             self.invoke, self.working_dir, self.path_args
         )
     }
 }
 
-fn replace_root(cmd: Vec<String>, root: &Path) -> Vec<String> {
+fn replace_root(cmd: &[String], root: &Path) -> Vec<String> {
     cmd.iter()
         .map(|c| {
             c.replace(
@@ -773,6 +908,7 @@ mod tests {
     use pretty_assertions::assert_eq;
     use serial_test::parallel;
     use std::env;
+    use test_case::test_case;
     use testhelper::TestHelper;
 
     fn matcher(globs: &[&str]) -> Result<Matcher> {
@@ -786,6 +922,7 @@ mod tests {
             name: String::new(),
             typ: LintOrTidyCommandType::Lint,
             includer: matcher(&[])?,
+            include: vec![],
             excluder: matcher(&[])?,
             invoke: Invoke::PerFile,
             working_dir: WorkingDir::Root,
@@ -809,17 +946,106 @@ mod tests {
             includer: matcher(&["**/*.go"])?,
             ..default_command()?
         };
-        let files = &["foo.go", "bar.go", "subdir/baz.go"]
+        let files = &["foo.go", "test/foo.go", "bar.go", "subdir/baz.go"]
             .iter()
             .map(PathBuf::from)
             .collect::<Vec<_>>();
+        let bar = PathBuf::from("bar.go");
+        let foo = PathBuf::from("foo.go");
+        let baz = PathBuf::from("subdir/baz.go");
+        let test_foo = PathBuf::from("test/foo.go");
         assert_eq!(
             command.files_to_args_sets(files)?,
-            vec![
-                vec![PathBuf::from("bar.go")],
-                vec![PathBuf::from("foo.go")],
-                vec![PathBuf::from("subdir/baz.go")],
-            ],
+            (
+                vec![
+                    vec![bar.as_path()],
+                    vec![foo.as_path()],
+                    vec![baz.as_path()],
+                    vec![test_foo.as_path()],
+                ],
+                ActualInvoke::PerFile,
+            ),
+        );
+
+        Ok(())
+    }
+
+    #[test]
+    #[parallel]
+    fn files_to_args_sets_per_file_or_dir() -> Result<()> {
+        let command = LintOrTidyCommand {
+            invoke: Invoke::PerFileOrDir(3),
+            includer: matcher(&["**/*.go"])?,
+            ..default_command()?
+        };
+        let files = &["foo.go", "test/foo.go", "bar.go", "subdir/baz.go"]
+            .iter()
+            .map(PathBuf::from)
+            .collect::<Vec<_>>();
+        let bar = PathBuf::from("bar.go");
+        let foo = PathBuf::from("foo.go");
+        let baz = PathBuf::from("subdir/baz.go");
+        let test_foo = PathBuf::from("test/foo.go");
+        assert_eq!(
+            command.files_to_args_sets(&files[0..2])?,
+            (
+                vec![vec![foo.as_path()], vec![test_foo.as_path()],],
+                ActualInvoke::PerFile,
+            ),
+            "with two paths invoke is PerFile",
+        );
+        assert_eq!(
+            command.files_to_args_sets(files)?,
+            (
+                vec![
+                    vec![bar.as_path(), foo.as_path()],
+                    vec![baz.as_path()],
+                    vec![test_foo.as_path()],
+                ],
+                ActualInvoke::PerDir,
+            ),
+            "with four paths invoke is PerDir",
+        );
+
+        Ok(())
+    }
+
+    #[test]
+    #[parallel]
+    fn files_to_args_sets_per_file_or_once() -> Result<()> {
+        let command = LintOrTidyCommand {
+            invoke: Invoke::PerFileOrOnce(3),
+            includer: matcher(&["**/*.go"])?,
+            ..default_command()?
+        };
+        let files = &["foo.go", "test/foo.go", "bar.go", "subdir/baz.go"]
+            .iter()
+            .map(PathBuf::from)
+            .collect::<Vec<_>>();
+        let bar = PathBuf::from("bar.go");
+        let foo = PathBuf::from("foo.go");
+        let baz = PathBuf::from("subdir/baz.go");
+        let test_foo = PathBuf::from("test/foo.go");
+        assert_eq!(
+            command.files_to_args_sets(&files[0..2])?,
+            (
+                vec![vec![foo.as_path()], vec![test_foo.as_path()],],
+                ActualInvoke::PerFile,
+            ),
+            "with 2 paths invoke is PerFile",
+        );
+        assert_eq!(
+            command.files_to_args_sets(files)?,
+            (
+                vec![vec![
+                    bar.as_path(),
+                    foo.as_path(),
+                    baz.as_path(),
+                    test_foo.as_path()
+                ]],
+                ActualInvoke::Once,
+            ),
+            "with four paths invoke is Once",
         );
 
         Ok(())
@@ -837,13 +1063,20 @@ mod tests {
             .iter()
             .map(PathBuf::from)
             .collect::<Vec<_>>();
+        let bar = PathBuf::from("bar.go");
+        let foo = PathBuf::from("foo.go");
+        let baz = PathBuf::from("subdir/baz.go");
+        let test_foo = PathBuf::from("test/foo.go");
         assert_eq!(
             command.files_to_args_sets(files)?,
-            vec![
-                vec![PathBuf::from("bar.go"), PathBuf::from("foo.go")],
-                vec![PathBuf::from("subdir/baz.go")],
-                vec![PathBuf::from("test/foo.go")],
-            ],
+            (
+                vec![
+                    vec![bar.as_path(), foo.as_path()],
+                    vec![baz.as_path()],
+                    vec![test_foo.as_path()],
+                ],
+                ActualInvoke::PerDir,
+            ),
         );
 
         Ok(())
@@ -857,17 +1090,25 @@ mod tests {
             includer: matcher(&["**/*.go"])?,
             ..default_command()?
         };
-        let files = ["foo.go", "bar.go", "subdir/baz.go"]
+        let files = &["foo.go", "test/foo.go", "bar.go", "subdir/baz.go"]
             .iter()
             .map(PathBuf::from)
             .collect::<Vec<_>>();
+        let bar = PathBuf::from("bar.go");
+        let foo = PathBuf::from("foo.go");
+        let baz = PathBuf::from("subdir/baz.go");
+        let test_foo = PathBuf::from("test/foo.go");
         assert_eq!(
-            command.files_to_args_sets(&files)?,
-            vec![vec![
-                PathBuf::from("bar.go"),
-                PathBuf::from("foo.go"),
-                PathBuf::from("subdir/baz.go"),
-            ],],
+            command.files_to_args_sets(files)?,
+            (
+                vec![vec![
+                    bar.as_path(),
+                    foo.as_path(),
+                    baz.as_path(),
+                    test_foo.as_path(),
+                ]],
+                ActualInvoke::Once,
+            ),
         );
 
         Ok(())
@@ -963,7 +1204,11 @@ mod tests {
         ];
         for i in include.iter().map(PathBuf::from) {
             let name = i.clone();
-            assert!(command.should_act_on_files(&[&i])?, "{}", name.display());
+            assert!(
+                command.should_act_on_files(ActualInvoke::PerFile, &[&i])?,
+                "{}",
+                name.display(),
+            );
         }
 
         let exclude = [
@@ -975,7 +1220,11 @@ mod tests {
         ];
         for e in exclude.iter().map(PathBuf::from) {
             let name = e.clone();
-            assert!(!command.should_act_on_files(&[&e])?, "{}", name.display());
+            assert!(
+                !command.should_act_on_files(ActualInvoke::PerFile, &[&e])?,
+                "{}",
+                name.display(),
+            );
         }
 
         Ok(())
@@ -1004,8 +1253,10 @@ mod tests {
         for i in include.iter() {
             let files = i.iter().map(PathBuf::from).collect::<Vec<_>>();
             assert!(
-                command
-                    .should_act_on_files(&files.iter().map(|f| f.as_ref()).collect::<Vec<_>>())?,
+                command.should_act_on_files(
+                    ActualInvoke::PerDir,
+                    &files.iter().map(|f| f.as_ref()).collect::<Vec<_>>(),
+                )?,
                 "{}",
                 i.join(", ")
             );
@@ -1020,8 +1271,10 @@ mod tests {
         for e in exclude.iter() {
             let files = e.iter().map(PathBuf::from).collect::<Vec<_>>();
             assert!(
-                !command
-                    .should_act_on_files(&files.iter().map(|f| f.as_ref()).collect::<Vec<_>>())?,
+                !command.should_act_on_files(
+                    ActualInvoke::PerDir,
+                    &files.iter().map(|f| f.as_ref()).collect::<Vec<_>>(),
+                )?,
                 "{}",
                 e.join(", ")
             );
@@ -1053,8 +1306,10 @@ mod tests {
             let files = i[1..].iter().map(PathBuf::from).collect::<Vec<PathBuf>>();
             let name = dir.clone();
             assert!(
-                command
-                    .should_act_on_files(&files.iter().map(|f| f.as_ref()).collect::<Vec<_>>())?,
+                command.should_act_on_files(
+                    ActualInvoke::Once,
+                    &files.iter().map(|f| f.as_ref()).collect::<Vec<_>>()
+                )?,
                 "{}",
                 name.display()
             );
@@ -1075,8 +1330,10 @@ mod tests {
             let files = e[1..].iter().map(PathBuf::from).collect::<Vec<PathBuf>>();
             let name = dir.clone();
             assert!(
-                !command
-                    .should_act_on_files(&files.iter().map(|f| f.as_ref()).collect::<Vec<_>>())?,
+                !command.should_act_on_files(
+                    ActualInvoke::Once,
+                    &files.iter().map(|f| f.as_ref()).collect::<Vec<_>>()
+                )?,
                 "{}",
                 name.display()
             );
@@ -1345,7 +1602,7 @@ mod tests {
         let mut file = helper.git_root();
         file.push("src/bar.rs");
         let metadata = command
-            .maybe_path_metadata_for(&[&file])?
+            .maybe_path_metadata_for(ActualInvoke::PerFile, &[&file])?
             .unwrap_or_else(|| unreachable!("Should always have metadata with Invoke::PerFile"));
         assert!(metadata.path_map.contains_key(&file));
 
@@ -1367,7 +1624,7 @@ mod tests {
         let mut dir = helper.git_root();
         dir.push("src");
         let metadata = command
-            .maybe_path_metadata_for(&[&dir])?
+            .maybe_path_metadata_for(ActualInvoke::PerFile, &[&dir])?
             .unwrap_or_else(|| unreachable!("Should always have metadata with Invoke::PerFile"));
         let expect_files = ["bar.rs", "main.rs", "module.rs"];
         for name in expect_files {
@@ -1393,7 +1650,9 @@ mod tests {
             ..default_command()?
         };
         let cwd = env::current_dir()?;
-        assert!(command.maybe_path_metadata_for(&[&cwd])?.is_none());
+        assert!(command
+            .maybe_path_metadata_for(ActualInvoke::Once, &[&cwd])?
+            .is_none());
 
         Ok(())
     }
@@ -1408,7 +1667,7 @@ mod tests {
         let paths = vec![PathBuf::from("app.go"), PathBuf::from("main.go")];
 
         assert_eq!(
-            command.command_for_paths(&None, &paths)?,
+            command.command_for_paths(&None, &paths),
             ["test", "app.go", "main.go"]
                 .iter()
                 .map(|s| s.to_string())
@@ -1418,7 +1677,7 @@ mod tests {
 
         let flags = vec![String::from("--flag")];
         assert_eq!(
-            command.command_for_paths(&Some(flags.clone()), &paths)?,
+            command.command_for_paths(&Some(flags.clone()), &paths),
             ["test", "--flag", "app.go", "main.go"]
                 .iter()
                 .map(|s| s.to_string())
@@ -1432,7 +1691,7 @@ mod tests {
             ..default_command()?
         };
         assert_eq!(
-            command.command_for_paths(&Some(flags), &paths)?,
+            command.command_for_paths(&Some(flags), &paths),
             [
                 "test",
                 "--flag",
@@ -1466,7 +1725,7 @@ mod tests {
         file.push("src/main.rs");
         let files = vec![file.as_ref()];
 
-        let prev = command.maybe_path_metadata_for(&files)?;
+        let prev = command.maybe_path_metadata_for(ActualInvoke::PerFile, &files)?;
         assert!(prev.is_some());
         assert!(!command.paths_were_changed(prev.clone().unwrap())?);
 
@@ -1492,7 +1751,7 @@ mod tests {
         file.push("src/main.rs");
         let files = vec![file.as_ref()];
 
-        let prev = command.maybe_path_metadata_for(&files)?;
+        let prev = command.maybe_path_metadata_for(ActualInvoke::PerFile, &files)?;
         assert!(prev.is_some());
         assert!(!command.paths_were_changed(prev.clone().unwrap())?);
 
@@ -1518,7 +1777,7 @@ mod tests {
         file.push("src/main.rs");
         let files = vec![file.as_ref()];
 
-        let prev = command.maybe_path_metadata_for(&files)?;
+        let prev = command.maybe_path_metadata_for(ActualInvoke::PerFile, &files)?;
         assert!(prev.is_some());
         assert!(!command.paths_were_changed(prev.clone().unwrap())?);
 
@@ -1555,8 +1814,10 @@ mod tests {
             }
         }
 
-        let prev = command
-            .maybe_path_metadata_for(&files.iter().map(|f| f.as_ref()).collect::<Vec<_>>())?;
+        let prev = command.maybe_path_metadata_for(
+            ActualInvoke::PerDir,
+            &files.iter().map(|f| f.as_ref()).collect::<Vec<_>>(),
+        )?;
         assert!(prev.is_some());
         let prev = prev.unwrap();
         assert_eq!(
@@ -1598,8 +1859,10 @@ mod tests {
             }
         }
 
-        let prev = command
-            .maybe_path_metadata_for(&files.iter().map(|f| f.as_ref()).collect::<Vec<_>>())?;
+        let prev = command.maybe_path_metadata_for(
+            ActualInvoke::PerDir,
+            &files.iter().map(|f| f.as_ref()).collect::<Vec<_>>(),
+        )?;
         assert!(prev.is_some());
         let prev = prev.unwrap();
         assert_eq!(
@@ -1614,4 +1877,106 @@ mod tests {
 
         Ok(())
     }
+
+    #[test_case(
+        ActualInvoke::Once,
+        &["**/*.go"],
+        &["foo.go"],
+        "foo.go";
+        "invoke = once with one path"
+    )]
+    #[test_case(
+        ActualInvoke::Once,
+        &["**/*.go"],
+        &["foo.go", "bar.go"],
+        "bar.go foo.go";
+        "invoke = once with two paths"
+    )]
+    #[test_case(
+        ActualInvoke::Once,
+        &["**/*.go"],
+        &["foo.go", "bar.go", "baz.go"],
+        "bar.go baz.go foo.go";
+        "invoke = once with three paths"
+    )]
+    #[test_case(
+        ActualInvoke::Once,
+        &["**/*.go"],
+        &["foo.go", "bar.go", "baz.go", "quux.go"],
+        "4 files matching **/*.go, starting with bar.go baz.go";
+        "invoke = once with four paths"
+    )]
+    #[test_case(
+        ActualInvoke::Once,
+        &["**/*.go", "!food.go"],
+        &["foo.go", "bar.go", "baz.go", "quux.go"],
+        "4 files matching **/*.go !food.go, starting with bar.go baz.go";
+        "invoke = once with four paths and two includes"
+    )]
+    #[test_case(
+        ActualInvoke::PerDir,
+        &["**/*.go"],
+        &["foo.go"],
+        "foo.go";
+        "invoke = dir with one path"
+    )]
+    #[test_case(
+        ActualInvoke::PerDir,
+        &["**/*.go"],
+        &["foo.go", "bar.go"],
+        "bar.go foo.go";
+        "invoke = dir with two paths"
+    )]
+    #[test_case(
+        ActualInvoke::PerDir,
+        &["**/*.go"],
+        &["foo.go", "bar.go", "baz.go"],
+        "bar.go baz.go foo.go";
+        "invoke = dir with three paths"
+    )]
+    #[test_case(
+        ActualInvoke::PerDir,
+        &["**/*.go"],
+        &["foo.go", "bar.go", "baz.go", "quux.go"],
+        "4 files matching **/*.go, starting with bar.go baz.go";
+        "invoke = dir with four paths"
+    )]
+    #[test_case(
+        ActualInvoke::PerDir,
+        &["**/*.go", "!food.go"],
+        &["foo.go", "bar.go", "baz.go", "quux.go"],
+        "4 files matching **/*.go !food.go, starting with bar.go baz.go";
+        "invoke = dir with four paths and two includes"
+    )]
+    #[test_case(
+        ActualInvoke::PerFile,
+        &["**/*.go", "!food.go"],
+        &["foo.go"],
+        "foo.go";
+        "invoke = file"
+    )]
+    #[parallel]
+    fn paths_summary(
+        actual_invoke: ActualInvoke,
+        include: &[&str],
+        paths: &[&str],
+        expect: &str,
+    ) -> Result<()> {
+        let command = LintOrTidyCommand {
+            name: String::from("Test"),
+            invoke: actual_invoke.as_invoke(),
+            include: include.iter().map(|i| i.to_string()).collect(),
+            ..default_command()?
+        };
+
+        assert_eq!(
+            &command.paths_summary(
+                actual_invoke,
+                &paths.iter().map(Path::new).collect::<Vec<_>>()
+            ),
+            expect,
+        );
+
+        Ok(())
+    }
 }
