@@ -29,6 +29,7 @@ pub struct Finder {
 }
 
 #[derive(Debug, Error, Eq, PartialEq)]
+#[allow(clippy::module_name_repetitions)]
 pub enum FinderError {
     #[error("You cannot pass an explicit list of files when looking for {mode:}")]
     GotPathsFromCliWithWrongMode { mode: Mode },
@@ -70,25 +71,33 @@ impl Finder {
             Mode::FromCli => (),
             _ => {
                 if !cli_paths.is_empty() {
-                    return Err(
-                        FinderError::GotPathsFromCliWithWrongMode { mode: self.mode }.into(),
-                    );
+                    return Err(FinderError::GotPathsFromCliWithWrongMode {
+                        mode: self.mode.clone(),
+                    }
+                    .into());
                 }
             }
         };
 
-        let mut files = match self.mode {
+        let mut files = match self.mode.clone() {
             Mode::All => self.all_files()?,
             Mode::FromCli => self.files_from_cli(cli_paths)?,
             Mode::GitModified => self.git_modified_files()?,
             Mode::GitStaged | Mode::GitStagedWithStash => self.git_staged_files()?,
+            Mode::GitDiffFrom(ref from) => self.git_modified_since(from)?,
         };
         files.sort();
 
         if files.is_empty() {
             return match self.mode {
-                Mode::GitModified | Mode::GitStaged | Mode::GitStagedWithStash => Ok(None),
-                _ => Err(FinderError::AllPathsWereExcluded { mode: self.mode }.into()),
+                Mode::GitModified
+                | Mode::GitStaged
+                | Mode::GitStagedWithStash
+                | Mode::GitDiffFrom(_) => Ok(None),
+                _ => Err(FinderError::AllPathsWereExcluded {
+                    mode: self.mode.clone(),
+                }
+                .into()),
             };
         }
 
@@ -97,7 +106,7 @@ impl Finder {
 
     fn git_root(&mut self) -> Result<PathBuf> {
         if let Some(r) = &self.git_root {
-            return Ok(r.to_path_buf());
+            return Ok(r.clone());
         }
 
         let res = exec::run(
@@ -185,16 +194,21 @@ impl Finder {
         Ok(())
     }
 
+    fn git_modified_since(&mut self, since: &str) -> Result<Vec<PathBuf>> {
+        let since_dot = format!("{since:}...");
+        self.files_from_git(&["diff", "--name-only", "--diff-filter=ACM", &since_dot])
+    }
+
     fn walkdir_files(&self, root: &Path) -> Result<Vec<PathBuf>> {
-        let mut excludes = ignore::overrides::OverrideBuilder::new(root);
+        let mut exclude_globs = ignore::overrides::OverrideBuilder::new(root);
         for d in vcs::DIRS {
-            excludes.add(&format!("!{d}/**/*"))?;
+            exclude_globs.add(&format!("!{d}/**/*"))?;
         }
 
         let mut files: Vec<PathBuf> = vec![];
         for result in ignore::WalkBuilder::new(root)
             .hidden(false)
-            .overrides(excludes.build()?)
+            .overrides(exclude_globs.build()?)
             .build()
         {
             match result {
@@ -246,6 +260,12 @@ impl Finder {
 
                             let mut f = git_root.clone();
                             f.push(&pb);
+                            if !f.exists() {
+                                debug!(
+                                    "The staged file at {rel:} was deleted so it will be ignored.",
+                                );
+                                return None;
+                            }
                             Some(f)
                         })
                         .collect(),
@@ -718,6 +738,44 @@ mod tests {
         Ok(())
     }
 
+    #[test]
+    #[parallel]
+    fn git_staged_mode_with_deleted_file() -> Result<()> {
+        let helper = testhelper::TestHelper::new()?.with_git_repo()?;
+        let mut modified = helper.modify_files()?;
+        helper.stage_all()?;
+        helper.delete_file(modified.remove(0))?;
+
+        let mut finder = new_finder(Mode::GitStaged, helper.precious_root())?;
+        assert_eq!(finder.files(vec![])?, Some(modified));
+        Ok(())
+    }
+
+    #[test]
+    #[parallel]
+    fn git_modified_since() -> Result<()> {
+        let helper = testhelper::TestHelper::new()?.with_git_repo()?;
+        helper.switch_to_branch("some-branch", false)?;
+
+        // When there are no commits in the branch the diff between master and
+        // the branch finds no files.
+        let mut finder = new_finder(
+            Mode::GitDiffFrom("master".to_string()),
+            helper.precious_root(),
+        )?;
+        assert_eq!(finder.files(vec![])?, None);
+
+        let modified = helper.modify_files()?;
+        helper.commit_all()?;
+
+        let mut finder = new_finder(
+            Mode::GitDiffFrom("master".to_string()),
+            helper.precious_root(),
+        )?;
+        assert_eq!(finder.files(vec![])?, Some(modified));
+        Ok(())
+    }
+
     #[test]
     #[parallel]
     fn cli_mode() -> Result<()> {
