@@ -1,27 +1,25 @@
 use crate::{
     chars,
-    command::{self, TidyOutcome},
-    config, config_init,
+    command::{self, ActualInvoke, TidyOutcome},
+    config,
+    config_init::{self, InitComponent},
     paths::{self, finder::Finder},
     vcs,
 };
 use anyhow::{Error, Result};
-use clap::{ArgGroup, Parser, ValueEnum};
+use clap::{ArgGroup, Parser};
 use comfy_table::{presets::UTF8_FULL, Cell, ContentArrangement, Table};
 use fern::{
     colors::{Color, ColoredLevelConfig},
     Dispatch,
 };
-use indexmap::{IndexMap, IndexSet};
 use itertools::Itertools;
 use log::{debug, error, info};
 use rayon::{prelude::*, ThreadPool, ThreadPoolBuilder};
 use std::{
-    collections::{HashMap, HashSet},
     env,
-    fs::{create_dir_all, File},
+    fmt::Write,
     io::stdout,
-    io::Write,
     path::{Path, PathBuf},
     time::{Duration, Instant},
 };
@@ -46,9 +44,6 @@ enum PreciousError {
 
     #[error("No {what:} commands match the given label, {label:}")]
     NoCommandsMatchLabel { what: String, label: String },
-
-    #[error("There is already a precious.toml file in this directory")]
-    ConfigFileExists,
 }
 
 #[derive(Debug)]
@@ -81,6 +76,7 @@ struct ActionFailure {
 #[clap(propagate_version = true)]
 #[clap(subcommand_required = true, arg_required_else_help = true)]
 #[clap(max_term_width = 100)]
+#[allow(clippy::struct_excessive_bools)]
 /// One code quality tool to rule them all
 pub struct App {
     /// Path to the precious config file
@@ -112,6 +108,7 @@ pub struct App {
 #[derive(Debug, Parser)]
 pub enum Subcommand {
     Lint(CommonArgs),
+    #[clap(alias = "fix")]
     Tidy(CommonArgs),
     Config(ConfigArgs),
 }
@@ -120,8 +117,9 @@ pub enum Subcommand {
 #[clap(group(
     ArgGroup::new("path-spec")
         .required(true)
-        .args(&["all", "git", "staged", "staged_with_stash", "paths"]),
+        .args(&["all", "git", "staged", "git_diff_from", "staged_with_stash", "paths"]),
 ))]
+#[allow(clippy::struct_excessive_bools)]
 pub struct CommonArgs {
     /// The command to run. If specified, only this command will be run. This
     /// should match the command name in your config file.
@@ -136,8 +134,16 @@ pub struct CommonArgs {
     /// Run against files that are staged for a git commit
     #[clap(long, short)]
     staged: bool,
+    /// Run against files that are different as compared with the given
+    /// `<REF>`. This can be a branch name, like `master`, or an ref name like
+    /// `HEAD~6` or `master@{2.days.ago}`. See `git help rev-parse` for more
+    /// options. Note that this will _not_ see files with uncommitted changes
+    /// in the local working directory.
+    #[clap(long, short = 'd', value_name = "REF")]
+    git_diff_from: Option<String>,
     /// Run against file content that is staged for a git commit, stashing all
-    /// unstaged content first
+    /// unstaged content first. The stash push/pop tends to do weird things to
+    /// the working directory, and is not recommended for scripting.
     #[clap(long)]
     staged_with_stash: bool,
     /// If this is set, then only commands matching this label will be run. If
@@ -163,28 +169,27 @@ enum ConfigSubcommand {
 }
 
 #[derive(Debug, Parser)]
+#[clap(group(
+    ArgGroup::new("components")
+        .required(true)
+        .args(&["component", "auto"]),
+))]
 pub struct ConfigInitArgs {
     #[clap(long, short, value_enum)]
     component: Vec<InitComponent>,
-    #[clap(long, short, default_value = "./precious.toml")]
+    #[clap(long, short)]
+    auto: bool,
+    #[clap(long, short, default_value = "precious.toml")]
     path: PathBuf,
 }
 
-#[derive(Clone, Copy, Debug, ValueEnum)]
-pub enum InitComponent {
-    Go,
-    Perl,
-    Rust,
-    Gitignore,
-    Markdown,
-    YAML,
-}
-
+#[must_use]
 pub fn app() -> App {
     App::parse()
 }
 
 impl App {
+    #[allow(clippy::missing_errors_doc)]
     pub fn init_logger(&self) -> Result<(), log::SetLoggerError> {
         let line_colors = ColoredLevelConfig::new()
             .error(Color::Red)
@@ -224,19 +229,24 @@ impl App {
             .apply()
     }
 
+    #[allow(clippy::missing_errors_doc)]
     pub fn run(self) -> Result<i8> {
         self._run(stdout())
     }
 
     #[cfg(test)]
-    fn run_with_output(self, output: impl Write) -> Result<i8> {
+    fn run_with_output(self, output: impl std::io::Write) -> Result<i8> {
         self._run(output)
     }
 
-    fn _run(self, output: impl Write) -> Result<i8> {
+    fn _run(self, output: impl std::io::Write) -> Result<i8> {
         if let Subcommand::Config(config_args) = &self.subcommand {
             if let ConfigSubcommand::Init(init_args) = &config_args.subcommand {
-                init_config(&init_args.component, &init_args.path)?;
+                config_init::write_config_files(
+                    init_args.auto,
+                    &init_args.component,
+                    &init_args.path,
+                )?;
                 return Ok(0);
             }
         }
@@ -271,7 +281,7 @@ impl App {
 
     fn load_config(&self) -> Result<(PathBuf, PathBuf, PathBuf, config::Config)> {
         let cwd = env::current_dir()?;
-        let project_root = project_root(self.config.as_ref(), &cwd)?;
+        let project_root = project_root(self.config.as_deref(), &cwd)?;
         let config_file = self.config_file(&project_root);
         let config = config::Config::new(&config_file)?;
 
@@ -281,7 +291,7 @@ impl App {
     fn config_file(&self, dir: &Path) -> PathBuf {
         if let Some(cf) = self.config.as_ref() {
             debug!("Loading config from {} (set via flag)", cf.display());
-            return cf.to_path_buf();
+            return cf.clone();
         }
 
         let default = default_config_file(dir);
@@ -293,7 +303,7 @@ impl App {
     }
 }
 
-fn project_root(config_file: Option<&PathBuf>, cwd: &Path) -> Result<PathBuf> {
+fn project_root(config_file: Option<&Path>, cwd: &Path) -> Result<PathBuf> {
     if let Some(file) = config_file {
         if let Some(p) = file.parent() {
             return Ok(p.to_path_buf());
@@ -308,9 +318,9 @@ fn project_root(config_file: Option<&PathBuf>, cwd: &Path) -> Result<PathBuf> {
         return Ok(cwd.into());
     }
 
-    for anc in cwd.ancestors() {
-        if is_checkout_root(anc) {
-            return Ok(anc.to_owned());
+    for ancestor in cwd.ancestors() {
+        if is_checkout_root(ancestor) {
+            return Ok(ancestor.to_owned());
         }
     }
 
@@ -336,7 +346,7 @@ fn default_config_file(dir: &Path) -> PathBuf {
             path.push(n);
             path
         }),
-        |p| p.exists(),
+        Path::exists,
     )
 }
 
@@ -363,7 +373,11 @@ fn is_checkout_root(dir: &Path) -> bool {
     false
 }
 
-fn print_config(mut output: impl Write, config_file: &Path, config: config::Config) -> Result<()> {
+fn print_config(
+    mut output: impl std::io::Write,
+    config_file: &Path,
+    config: config::Config,
+) -> Result<()> {
     writeln!(output, "Found config file at: {}", config_file.display())?;
     writeln!(output)?;
 
@@ -389,106 +403,6 @@ fn print_config(mut output: impl Write, config_file: &Path, config: config::Conf
     Ok(())
 }
 
-fn init_config(components: &[InitComponent], path: &Path) -> Result<()> {
-    if has_config_file(&env::current_dir()?) {
-        return Err(PreciousError::ConfigFileExists.into());
-    }
-
-    let mut excludes: HashSet<&'static str> = HashSet::new();
-    let mut commands = IndexMap::new();
-    let mut extra_files = HashMap::new();
-    let mut tool_urls: IndexSet<&'static str> = IndexSet::new();
-
-    for l in components {
-        let init = match l {
-            InitComponent::Go => config_init::go_init(),
-            InitComponent::Perl => config_init::perl_init(),
-            InitComponent::Rust => config_init::rust_init(),
-            InitComponent::Gitignore => config_init::gitignore_init(),
-            InitComponent::Markdown => config_init::markdown_init(),
-            InitComponent::YAML => config_init::yaml_init(),
-        };
-        excludes.extend(init.excludes);
-        for (name, c) in init.commands {
-            commands.insert(name, c);
-        }
-        for (name, c) in init.extra_files {
-            extra_files.insert(name, c);
-        }
-        tool_urls.extend(init.tool_urls);
-    }
-
-    let mut toml = String::new();
-    if !excludes.is_empty() {
-        toml.push_str(&format!(
-            "excludes = [\n{}\n]",
-            excludes
-                .iter()
-                .sorted()
-                .map(|e| format!(r#"    "{e}","#))
-                .collect::<Vec<_>>()
-                .join("\n")
-        ));
-    }
-
-    if !toml.is_empty() {
-        toml.push_str("\n\n");
-    }
-
-    let mut command_strs: Vec<String> = Vec::new();
-
-    for (name, c) in commands {
-        let name_str = if name.contains(' ') {
-            format!(r#""{name}""#)
-        } else {
-            name.to_string()
-        };
-        command_strs.push(format!("[commands.{name_str}]\n{}\n", c.trim()));
-    }
-
-    toml.push_str(&command_strs.join("\n"));
-
-    println!();
-    println!("Writing {}", path.display());
-
-    let mut precious_toml = File::create(path)?;
-    precious_toml.write_all(toml.as_bytes())?;
-
-    if !extra_files.is_empty() {
-        println!();
-        println!("Generating support files");
-        println!();
-
-        let paths = extra_files.keys().sorted().collect::<Vec<_>>();
-
-        for p in paths {
-            print!("{} ...", p.display());
-            if p.exists() {
-                println!(
-                    "  already exists, skipping - delete this file if you want to regenerate it"
-                );
-                continue;
-            }
-            println!(" generated");
-
-            if let Some(parent) = p.parent() {
-                create_dir_all(parent)?;
-            }
-            let mut file = File::create(p)?;
-            file.write_all(extra_files.get(p).unwrap().trim_start().as_bytes())?;
-        }
-    }
-
-    println!();
-    println!("The generated precious.toml requires the following tools to be installed:");
-    for u in tool_urls {
-        println!("  {u}");
-    }
-    println!();
-
-    Ok(())
-}
-
 #[derive(Debug)]
 pub struct LintOrTidyRunner {
     mode: paths::mode::Mode,
@@ -505,7 +419,7 @@ pub struct LintOrTidyRunner {
 }
 
 impl LintOrTidyRunner {
-    pub fn new(
+    fn new(
         app: App,
         cwd: PathBuf,
         project_root: PathBuf,
@@ -549,8 +463,7 @@ impl LintOrTidyRunner {
 
     fn mode(app: &App) -> Result<paths::mode::Mode> {
         let common = match &app.subcommand {
-            Subcommand::Lint(c) => c,
-            Subcommand::Tidy(c) => c,
+            Subcommand::Lint(c) | Subcommand::Tidy(c) => c,
             Subcommand::Config(_) => unreachable!("this is handled in App::run"),
         };
         if common.all {
@@ -559,9 +472,12 @@ impl LintOrTidyRunner {
             return Ok(paths::mode::Mode::GitModified);
         } else if common.staged {
             return Ok(paths::mode::Mode::GitStaged);
+        } else if let Some(from) = &common.git_diff_from {
+            return Ok(paths::mode::Mode::GitDiffFrom(from.clone()));
         } else if common.staged_with_stash {
             return Ok(paths::mode::Mode::GitStagedWithStash);
         }
+
         if common.paths.is_empty() {
             return Err(PreciousError::NoModeOrPathsInCliArgs.into());
         }
@@ -582,7 +498,7 @@ impl LintOrTidyRunner {
             }
             Err(e) => {
                 error!("Failed to run precious: {}", e);
-                1
+                42
             }
         }
     }
@@ -678,31 +594,31 @@ impl LintOrTidyRunner {
         };
 
         match self.finder()?.files(cli_paths)? {
-            None => Ok(self.no_files_exit()),
+            None => Ok(Self::no_files_exit()),
             Some(files) => {
                 let mut all_failures: Vec<ActionFailure> = vec![];
                 for c in commands {
-                    debug!(r#"Command config for {}: {}"#, c.name, c.config_debug(),);
+                    debug!(r#"Command config for {}: {}"#, c.name, c.config_debug());
                     if let Some(mut failures) = run_command(self, &files, &c)? {
                         all_failures.append(&mut failures);
                     }
                 }
 
-                Ok(self.make_exit(all_failures, action))
+                Ok(self.make_exit(&all_failures, action))
             }
         }
     }
 
     fn finder(&mut self) -> Result<Finder> {
         Finder::new(
-            self.mode,
+            self.mode.clone(),
             self.project_root.clone(),
             self.cwd.clone(),
             self.config.exclude.clone(),
         )
     }
 
-    fn make_exit(&self, failures: Vec<ActionFailure>, action: &str) -> Exit {
+    fn make_exit(&self, failures: &[ActionFailure], action: &str) -> Exit {
         let (status, error) = if failures.is_empty() {
             (0, None)
         } else {
@@ -716,17 +632,17 @@ impl LintOrTidyRunner {
                 plural,
                 action,
                 ansi_off,
-                failures
-                    .iter()
-                    .map(|af| format!(
-                        "  {} [{}] [{}]\n    {}\n",
+                failures.iter().fold(String::new(), |mut out, af| {
+                    let _ = write!(
+                        out,
+                        "  {} [{}] failed for [{}]\n    {}\n",
                         self.chars.bullet,
-                        af.paths.iter().map(|p| p.to_string_lossy()).join(" "),
                         af.config_key,
+                        af.paths.iter().map(|p| p.to_string_lossy()).join(" "),
                         af.error,
-                    ))
-                    .collect::<Vec<String>>()
-                    .join("")
+                    );
+                    out
+                }),
             );
             (1, Some(error))
         };
@@ -742,15 +658,18 @@ impl LintOrTidyRunner {
         files: &[PathBuf],
         t: &command::LintOrTidyCommand,
     ) -> Result<Option<Vec<ActionFailure>>> {
-        let runner = |s: &Self, files: &[&Path]| -> Option<Result<(), ActionFailure>> {
-            match t.tidy(files) {
+        let runner = |s: &Self,
+                      actual_invoke: ActualInvoke,
+                      files: &[&Path]|
+         -> Option<Result<(), ActionFailure>> {
+            match t.tidy(actual_invoke, files) {
                 Ok(Some(TidyOutcome::Changed)) => {
                     if !s.quiet {
                         println!(
-                            "{} Tidied by {}:    [{}]",
+                            "{} Tidied by {}:    {}",
                             s.chars.tidied,
                             t.name,
-                            files.iter().map(|p| p.to_string_lossy()).join(" "),
+                            t.paths_summary(actual_invoke, files),
                         );
                     }
                     Some(Ok(()))
@@ -758,10 +677,10 @@ impl LintOrTidyRunner {
                 Ok(Some(TidyOutcome::Unchanged)) => {
                     if !s.quiet {
                         println!(
-                            "{} Unchanged by {}: [{}]",
+                            "{} Unchanged by {}: {}",
                             s.chars.unchanged,
                             t.name,
-                            files.iter().map(|p| p.to_string_lossy()).join(" "),
+                            t.paths_summary(actual_invoke, files),
                         );
                     }
                     Some(Ok(()))
@@ -769,10 +688,10 @@ impl LintOrTidyRunner {
                 Ok(Some(TidyOutcome::Unknown)) => {
                     if !s.quiet {
                         println!(
-                            "{} Maybe changed by {}: [{}]",
+                            "{} Maybe changed by {}: {}",
                             s.chars.unknown,
                             t.name,
-                            files.iter().map(|p| p.to_string_lossy()).join(" "),
+                            t.paths_summary(actual_invoke, files),
                         );
                     }
                     Some(Ok(()))
@@ -780,10 +699,10 @@ impl LintOrTidyRunner {
                 Ok(None) => None,
                 Err(e) => {
                     println!(
-                        "{} Error from {}: [{}]",
+                        "{} Error from {}: {}",
                         s.chars.execution_error,
                         t.name,
-                        files.iter().map(|p| p.to_string_lossy()).join(" "),
+                        t.paths_summary(actual_invoke, files),
                     );
                     Some(Err(ActionFailure {
                         error: format!("{e:#}"),
@@ -802,8 +721,11 @@ impl LintOrTidyRunner {
         files: &[PathBuf],
         l: &command::LintOrTidyCommand,
     ) -> Result<Option<Vec<ActionFailure>>> {
-        let runner = |s: &Self, files: &[&Path]| -> Option<Result<(), ActionFailure>> {
-            match l.lint(files) {
+        let runner = |s: &Self,
+                      actual_invoke: ActualInvoke,
+                      files: &[&Path]|
+         -> Option<Result<(), ActionFailure>> {
+            match l.lint(actual_invoke, files) {
                 Ok(Some(lo)) => {
                     if lo.ok {
                         if !s.quiet {
@@ -811,7 +733,7 @@ impl LintOrTidyRunner {
                                 "{} Passed {}: {}",
                                 s.chars.lint_free,
                                 l.name,
-                                files.iter().map(|p| p.to_string_lossy()).join(" "),
+                                l.paths_summary(actual_invoke, files),
                             );
                         }
                         Some(Ok(()))
@@ -820,7 +742,7 @@ impl LintOrTidyRunner {
                             "{} Failed {}: {}",
                             s.chars.lint_dirty,
                             l.name,
-                            files.iter().map(|p| p.to_string_lossy()).join(" "),
+                            l.paths_summary(actual_invoke, files),
                         );
                         if let Some(s) = lo.stdout {
                             println!("{s}");
@@ -855,7 +777,7 @@ impl LintOrTidyRunner {
                         "{} error {}: {}",
                         s.chars.execution_error,
                         l.name,
-                        files.iter().map(|p| p.to_string_lossy()).join(" "),
+                        l.paths_summary(actual_invoke, files),
                     );
                     Some(Err(ActionFailure {
                         error: format!("{e:#}"),
@@ -877,9 +799,9 @@ impl LintOrTidyRunner {
         runner: R,
     ) -> Result<Option<Vec<ActionFailure>>>
     where
-        R: Fn(&Self, &[&Path]) -> Option<Result<(), ActionFailure>> + Sync,
+        R: Fn(&Self, ActualInvoke, &[&Path]) -> Option<Result<(), ActionFailure>> + Sync,
     {
-        let sets = c.files_to_args_sets(files)?;
+        let (sets, actual_invoke) = c.files_to_args_sets(files)?;
 
         let start = Instant::now();
         let results = self
@@ -889,7 +811,7 @@ impl LintOrTidyRunner {
                 res.append(
                     &mut sets
                         .into_par_iter()
-                        .filter_map(|set| runner(self, &set))
+                        .filter_map(|set| runner(self, actual_invoke, &set))
                         .collect::<Vec<Result<(), ActionFailure>>>(),
                 );
                 Ok(res)
@@ -909,7 +831,7 @@ impl LintOrTidyRunner {
         let failures = results
             .into_iter()
             .filter_map(|r| match r {
-                Ok(_) => None,
+                Ok(()) => None,
                 Err(e) => Some(e),
             })
             .collect::<Vec<ActionFailure>>();
@@ -920,7 +842,7 @@ impl LintOrTidyRunner {
         }
     }
 
-    fn no_files_exit(&self) -> Exit {
+    fn no_files_exit() -> Exit {
         Exit {
             status: 0,
             message: Some(String::from("No files found")),
@@ -938,6 +860,11 @@ impl LintOrTidyRunner {
 //    X.XXms
 //    X.XXus
 //    X.XXns
+#[allow(
+    clippy::cast_possible_truncation,
+    clippy::cast_precision_loss,
+    clippy::cast_sign_loss
+)]
 fn format_duration(d: &Duration) -> String {
     let s = (d.as_secs_f64() * 100.0).round() / 100.0;
 
@@ -963,9 +890,9 @@ fn format_duration(d: &Duration) -> String {
 mod tests {
     use super::*;
     use itertools::Itertools;
-    use precious_helpers::pushd::Pushd;
     use precious_testhelper::TestHelper;
     use pretty_assertions::assert_eq;
+    use pushd::Pushd;
     // Anything that does pushd must be run serially or else chaos ensues.
     use serial_test::serial;
     #[cfg(not(target_os = "windows"))]
@@ -980,9 +907,9 @@ mod tests {
 type    = "both"
 include = "**/*.rs"
 cmd     = ["rustfmt"]
-lint_flags = "--check"
-ok_exit_codes = [0]
-lint_failure_exit_codes = [1]
+lint-flags = "--check"
+ok-exit-codes = [0]
+lint-failure-exit-codes = [1]
 "#;
 
     const DEFAULT_CONFIG_FILE_NAME: &str = super::CONFIG_FILE_NAMES[0];
@@ -1176,7 +1103,7 @@ lint_failure_exit_codes = [1]
     type    = "tidy"
     include = "**/*"
     cmd     = ["true"]
-    ok_exit_codes = [0]
+    ok-exit-codes = [0]
     "#;
         let helper = TestHelper::new()?.with_config_file(DEFAULT_CONFIG_FILE_NAME, config)?;
         let _pushd = helper.pushd_to_git_root()?;
@@ -1200,7 +1127,7 @@ lint_failure_exit_codes = [1]
     type    = "tidy"
     include = "**/*"
     cmd     = ["false"]
-    ok_exit_codes = [0]
+    ok-exit-codes = [0]
     "#;
         let helper = TestHelper::new()?.with_config_file(DEFAULT_CONFIG_FILE_NAME, config)?;
         let _pushd = helper.pushd_to_git_root()?;
@@ -1224,8 +1151,8 @@ lint_failure_exit_codes = [1]
     type    = "lint"
     include = "**/*"
     cmd     = ["true"]
-    ok_exit_codes = [0]
-    lint_failure_exit_codes = [1]
+    ok-exit-codes = [0]
+    lint-failure-exit-codes = [1]
     "#;
         let helper = TestHelper::new()?.with_config_file(DEFAULT_CONFIG_FILE_NAME, config)?;
         let _pushd = helper.pushd_to_git_root()?;
@@ -1283,7 +1210,7 @@ lint_failure_exit_codes = [1]
         let mut lt = app.new_lint_or_tidy_runner()?;
         let status = lt.run();
 
-        assert_eq!(status, 1);
+        assert_eq!(status, 42);
 
         Ok(())
     }
@@ -1306,21 +1233,21 @@ lint_failure_exit_codes = [1]
             type    = "tidy"
             include = "test.replace"
             cmd     = ["perl", "-pi", "-e", "s/a/b/i"]
-            ok_exit_codes = [0]
+            ok-exit-codes = [0]
 
             [commands.perl-replace-a-with-c]
             type    = "tidy"
             include = "test.replace"
             cmd     = ["perl", "-pi", "-e", "s/a/c/i"]
-            ok_exit_codes = [0]
-            lint_failure_exit_codes = [1]
+            ok-exit-codes = [0]
+            lint-failure-exit-codes = [1]
 
             [commands.perl-replace-a-with-d]
             type    = "tidy"
             include = "test.replace"
             cmd     = ["perl", "-pi", "-e", "s/a/d/i"]
-            ok_exit_codes = [0]
-            lint_failure_exit_codes = [1]
+            ok-exit-codes = [0]
+            lint-failure-exit-codes = [1]
         "#;
         let helper = TestHelper::new()?.with_config_file(DEFAULT_CONFIG_FILE_NAME, config)?;
         let test_replace = PathBuf::from_str("test.replace")?;
@@ -1346,20 +1273,20 @@ lint_failure_exit_codes = [1]
             type    = "lint"
             include = "*.foo"
             cmd     = ["foo", "--lint", "--with-vigor"]
-            ok_exit_codes = [0]
+            ok-exit-codes = [0]
 
             [commands.bar]
             type    = "tidy"
             include = "*.bar"
             cmd     = ["bar", "--fix-broken-things", "--aggressive"]
-            ok_exit_codes = [0]
+            ok-exit-codes = [0]
 
             [commands.baz]
             type    = "both"
             include = "*.baz"
             cmd     = ["baz", "--fast-mode", "--no-verify"]
-            lint_flags = "--lint"
-            ok_exit_codes = [0]
+            lint-flags = "--lint"
+            ok-exit-codes = [0]
         "#;
         let helper = TestHelper::new()?.with_config_file(DEFAULT_CONFIG_FILE_NAME, config)?;
         let _pushd = helper.pushd_to_git_root()?;
