@@ -1,7 +1,8 @@
 use anyhow::{Context, Result};
 use log::debug;
 use once_cell::sync::{Lazy, OnceCell};
-use precious_helpers::{exec, pushd::Pushd};
+use precious_helpers::exec;
+use pushd::Pushd;
 use regex::Regex;
 use std::{
     collections::HashMap,
@@ -118,7 +119,7 @@ impl TestHelper {
     }
 
     pub fn pushd_to_git_root(&self) -> Result<Pushd> {
-        pushd_to(self.git_root.clone())
+        Ok(Pushd::new(self.git_root.clone())?)
     }
 
     pub fn git_root(&self) -> PathBuf {
@@ -267,6 +268,17 @@ generated.*
         Ok(())
     }
 
+    pub fn delete_file<P: AsRef<Path>>(&self, rel: P) -> Result<()> {
+        let mut full = self.precious_root.clone();
+        full.push(rel.as_ref());
+        debug!("deleting path at {}", full.display());
+        if full.is_file() {
+            return Ok(fs::remove_file(full)?);
+        }
+
+        Ok(fs::remove_dir_all(full)?)
+    }
+
     #[cfg(not(target_os = "windows"))]
     pub fn read_file(&self, rel: &Path) -> Result<String> {
         let mut full = self.precious_root.clone();
@@ -278,10 +290,6 @@ generated.*
     }
 }
 
-pub fn pushd_to(to: PathBuf) -> Result<Pushd> {
-    Pushd::new(to)
-}
-
 fn is_rust_file(p: &Path) -> bool {
     if let Some(e) = p.extension() {
         let rs = OsString::from("rs");
